//////////////////////////////////////////////////////////////////////////////////
// :   
// :  .. anis@pmg-ru.narod.ru
// : 1.00
// : 06.2001
//////////////////////////////////////////////////////////////////////////////////
#include "rm.h"
#include "enumerate.h"
#include "scene.h"
#include <stdio.h>
#include <time.h>
#include <mmsystem.h>
#include <math.h>

//  
BOOL build_fan
	( LPDIRECT3DRMMESHBUILDER3 mb,
	  D3DVECTOR & beg_point,
	  D3DVALUE radius, D3DVALUE height, D3DVALUE y_normal,
	  int max_point )
{
	int i;
	D3DVALUE x, y, z, delta, beg;
	VertexTriangle_t * tri_fan;

	if ( mb==0 ) return FALSE;
	if ( max_point%2 ) max_point++;
	//   
	tri_fan=(VertexTriangle_t *)malloc(sizeof(VertexTriangle_t)*(max_point+2));
	if ( tri_fan==0 ) return FALSE;
	
	//  
	tri_fan[0].type=D3DRMVERTEX_FAN;
	SET_D3DVECTOR ( tri_fan[0].pos, beg_point.x, beg_point.y, beg_point.z);
	SET_D3DVECTOR ( tri_fan[0].normal, 0.0f, y_normal, 0.0f);
	//  .      
	if ( y_normal>0 ) {	beg=0; delta=-2*DEGREES_180/max_point;	}
	else { beg=2*DEGREES_180; delta=2*DEGREES_180/max_point; }
	//  Y 
	y=beg_point.y+height;
	for ( i=0; i<max_point+1; i++ )
	{
		tri_fan[i+1].type=D3DRMVERTEX_FAN;
		//    
		x=D3DVAL(radius*sin(beg-(i*delta)));
		z=D3DVAL(radius*cos(beg-(i*delta)));
		SET_D3DVECTOR ( tri_fan[i+1].pos, beg_point.x+x, y, beg_point.z+z);
		//  ,    ,   
		//      
		if ( height==0 )
			SET_D3DVECTOR ( tri_fan[i+1].normal, 0, y_normal, 0)
		else
			SET_D3DVECTOR ( tri_fan[i+1].normal, x, y_normal, z)
		D3DRMVectorNormalize(&tri_fan[i+1].normal);
	}
	//  
    mb->AddTriangles( 0, D3DRMFVF_TYPE | D3DRMFVF_NORMAL, max_point+2, (LPVOID)tri_fan );
	free(tri_fan);
	return TRUE;
}

//     
BOOL build_cone(LPDIRECT3DRMMESHBUILDER3 mb, int max_point)
{
	D3DVECTOR point;

	//  
	mb->ReserveSpace(2*(max_point+2),2*(max_point+2),2*(max_point+2));
	//  
	SET_D3DVECTOR (point, 0,0.5,0 );
	if ( build_fan(mb,point,D3DVAL(1),D3DVAL(-1),D3DVAL(0.5),max_point)==FALSE )
		return FALSE;
	// 
	SET_D3DVECTOR (point, 0,-0.5,0 );
	if ( build_fan(mb,point,D3DVAL(1),D3DVAL(0),D3DVAL(-0.5),max_point)==FALSE )
		return FALSE;
	return TRUE;
}


//   
BOOL build_cylinder_strip
	( LPDIRECT3DRMMESHBUILDER3 mb,
	  D3DVECTOR & top_point, D3DVECTOR & end_point,
	  D3DVALUE top_radius, D3DVALUE end_radius,
	  int max_point )
{
	int i;
	D3DVALUE x, y_top, y_end, z, delta;
	VertexTriangle_t * tri_strip;

	if ( mb==0 ) return FALSE;
	if ( max_point%2 ) max_point++;
	//   
	tri_strip=(VertexTriangle_t *)malloc(sizeof(VertexTriangle_t)*(2*(max_point+1)));
	if ( tri_strip==0 ) return FALSE;
	
	//   
	delta=2*DEGREES_180/max_point;
	//  Y 
	y_top=top_point.y;
	y_end=end_point.y;
	for ( i=0; i<max_point+1; i++ )
	{
		//  
		tri_strip[2*i].type=D3DRMVERTEX_STRIP;
		//    
		x=D3DVAL(top_radius*sin(i*delta));
		z=D3DVAL(top_radius*cos(i*delta));
		SET_D3DVECTOR ( tri_strip[2*i].pos, x, y_top, z)
		//  
		SET_D3DVECTOR ( tri_strip[2*i].normal, x, y_top, z)
		D3DRMVectorNormalize(&tri_strip[2*i].normal);

		//  
		tri_strip[2*i+1].type=D3DRMVERTEX_STRIP;
		//    
		x=D3DVAL(end_radius*sin(i*delta));
		z=D3DVAL(end_radius*cos(i*delta));
		SET_D3DVECTOR ( tri_strip[2*i+1].pos, x, y_end, z)
		//  
		SET_D3DVECTOR ( tri_strip[2*i+1].normal, x, y_end, z)
		D3DRMVectorNormalize(&tri_strip[2*i+1].normal);
	}
	//  
    mb->AddTriangles( 0, D3DRMFVF_TYPE | D3DRMFVF_NORMAL, 2*(max_point+1), (LPVOID)tri_strip );
	free(tri_strip);
	return TRUE;
}


//  
BOOL build_cylinder(LPDIRECT3DRMMESHBUILDER3 mb, int max_point)
{
	D3DVECTOR top_point, end_point;

	//  
	mb->ReserveSpace(3*(max_point+2),3*(max_point+2),3*(max_point+2));

	SET_D3DVECTOR (top_point, 0,0.5,0 );
	SET_D3DVECTOR (end_point, 0,-0.5,0 );
	//  
	if ( build_fan(mb,top_point,D3DVAL(0.5),D3DVAL(0),D3DVAL(0.5),max_point)==FALSE )
		return FALSE;
	//  
	if ( build_cylinder_strip(
		mb, top_point,end_point, D3DVAL(0.5),D3DVAL(0.5),max_point)==FALSE )
		return FALSE;
	//  
	SET_D3DVECTOR (end_point, 0,-0.5,0 );
	if ( build_fan(mb,end_point,D3DVAL(0.5),D3DVAL(0),D3DVAL(-0.5),max_point)==FALSE )
		return FALSE;

	return TRUE;

}

//  
BOOL build_sphere(LPDIRECT3DRMMESHBUILDER3 mb, int max_point_xz, int max_point_y)
{
	D3DVECTOR top_point, end_point;
	double delta, y, r, y1, r1, radius=0.5;
	int i;

	//  
	mb->ReserveSpace(
		2*(max_point_xz+2)+2*(max_point_y+1),
		2*(max_point_xz+2)+2*(max_point_y+1),
		2*(max_point_xz+2)+2*(max_point_y+1));

	delta=DEGREES_180/max_point_y;

	//  
	SET_D3DVECTOR (top_point, 0,radius,0 );
	i=1; r=radius*sin(i*delta); y=radius*cos(i*delta);
	if ( build_fan(mb,top_point,D3DVAL(r),D3DVAL(y-radius),D3DVAL(radius),max_point_xz)==FALSE )
		return FALSE;

	//  
	for ( i=1; i<max_point_y-1; i++ )
	{
		r=radius*sin(i*delta); r1=radius*sin((i+1)*delta);
		y=radius*cos(i*delta); y1=radius*cos((i+1)*delta);
		SET_D3DVECTOR (top_point, 0,y,0 );
		SET_D3DVECTOR (end_point, 0,y1,0 );
		if ( build_cylinder_strip(
			mb, top_point,end_point, D3DVAL(r),D3DVAL(r1),max_point_xz)==FALSE )
		return FALSE;
	}

	//  
	SET_D3DVECTOR (end_point, 0,-radius,0 );
	i=max_point_y-1; r=radius*sin(i*delta); y=radius*cos(i*delta);
	if ( build_fan(mb,end_point,D3DVAL(r),D3DVAL(radius+y),D3DVAL(-radius),max_point_xz)==FALSE )
		return FALSE;

	return TRUE;

}

//   : , , 
BOOL DemoShapesRotate()
{
    LPDIRECT3DRMFRAME3       frame=0;
    LPDIRECT3DRMMESHBUILDER3 builder=0;
    LPDIRECT3DRMMESHBUILDER3 save_builder=0;
	LPDIRECT3DRMMESH mesh=0;
	HRESULT rc;

	//    
	rc=rm_info.rm->CreateMeshBuilder(&save_builder);
	DD_CHK(rc,_gError);

	//  
	// 
	rc=rm_info.rm->CreateFrame( main_frame, &frame );
	DD_CHK(rc,_gError);
	// 
	frame->SetPosition( main_frame, D3DVAL(0), D3DVAL(0), D3DVAL(0) );
	rc=rm_info.rm->CreateMeshBuilder(&builder);
	DD_CHK(rc,_gError);
	//  
	build_cone(builder,20);
	//   
	//builder->SetQuality(D3DRMFILL_WIREFRAME);
	// 
	builder->SetColor(RGBA_MAKE(0,255,255,200));
	// 
	builder->Scale(D3DVAL(10),D3DVAL(20),D3DVAL(10));
	//  
	builder->CreateMesh(&mesh);
	//   
	frame->AddVisual( (LPDIRECT3DRMVISUAL)mesh );
	save_builder->AddMeshBuilder(builder,0);
	save_builder->Translate(D3DVAL(-20), D3DVAL(0), D3DVAL(0));
	DD_RELEASE( frame );
	DD_RELEASE( mesh );
	DD_RELEASE( builder );

	//  
	// 
	rc=rm_info.rm->CreateFrame( main_frame, &frame );
	DD_CHK(rc,_gError);
	// 
	frame->SetPosition( main_frame, D3DVAL(20), D3DVAL(0), D3DVAL(0) );
	rc=rm_info.rm->CreateMeshBuilder(&builder);
	DD_CHK(rc,_gError);
	//  
	build_cylinder(builder,20);
	//   
	//builder->SetQuality(D3DRMFILL_WIREFRAME);
	// 
	builder->SetColor(RGBA_MAKE(0,255,255,200));
	// 
	builder->Scale(D3DVAL(10),D3DVAL(20),D3DVAL(10));
	//  
	builder->CreateMesh(&mesh);
	//   
	frame->AddVisual( (LPDIRECT3DRMVISUAL)mesh );
	save_builder->AddMeshBuilder(builder,0);
	save_builder->Translate(D3DVAL(-20), D3DVAL(0), D3DVAL(0));
	DD_RELEASE( frame );
	DD_RELEASE( mesh );
	DD_RELEASE( builder );

	//  
	// 
	rc=rm_info.rm->CreateFrame( main_frame, &frame );
	DD_CHK(rc,_gError);
	// 
	frame->SetPosition( main_frame, -20.0f, 0.0f, 0.0f );
	rc=rm_info.rm->CreateMeshBuilder(&builder);
	DD_CHK(rc,_gError);
	//  
	build_sphere(builder,20,20);
	//   
	//builder->SetQuality(D3DRMFILL_WIREFRAME);
	// 
	builder->SetColor(RGBA_MAKE(0,255,255,200));
	// 
	//ScaleMesh(builder,D3DVAL(20));
	builder->Scale(D3DVAL(10),D3DVAL(20),D3DVAL(5));
	//  
	builder->CreateMesh(&mesh);
	//   
	frame->AddVisual( (LPDIRECT3DRMVISUAL)mesh );
	save_builder->AddMeshBuilder(builder,0);
	DD_RELEASE( frame );
	DD_RELEASE( mesh );
	DD_RELEASE( builder );

	save_builder->Translate(D3DVAL(20), D3DVAL(0), D3DVAL(0));
	//save_builder->Save("rotate.x",D3DRMXOF_TEXT,D3DRMXOFSAVE_NORMALS);
	DD_RELEASE( save_builder );

	return TRUE;

_gError:
	DD_RELEASE( frame );
	DD_RELEASE( mesh );
	DD_RELEASE( builder );
	DD_RELEASE( save_builder );
	return FALSE;
}
